// -*- C++ -*-
/*!
 * @file  BirdseyeImageViewer.h
 * @brief Birdseye Image Viewer component
 * @date  $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * $Id$
 */

#ifndef BIRDSEYEIMAGEVIEWER_H
#define BIRDSEYEIMAGEVIEWER_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <iostream>
#include <string>
#include <windows.h>
#include "local_basic_type.h"
/*!
 * @class BirdseyeImageViewer
 * @brief Birdseye Image Viewer component
 *
 * [機能仕様書_俯瞰画像ビューアRTC]参照
 *
 * [機能仕様書_俯瞰画像ビューアRTC]参照
 *
 * [機能仕様書_俯瞰画像ビューアRTC]参照
 *
 */
class BirdseyeImageViewer
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  BirdseyeImageViewer(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~BirdseyeImageViewer();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * Initialization of Viewer RTC
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre nothing special
   * @post nothing special
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * 
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * Activate the Viewer RTC
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre nothing special
   * @post nothing special
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * Deactivate the Viewer RTC
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre nothing special
   * @post nothing special
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * Periodic task of Veiwer RTC. This task read the image file.
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre nothing special
   * @post nothing special
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * WindowAの表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: windowA_showmode A_showmode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_A_showmode;
  /*!
   * WindowBの表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: windowB_showmode B_showmode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_B_showmode;
  /*!
   * WindowCの表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: windowC_showmode C_showmode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_C_showmode;
  /*!
   * WindowDの表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: windowD_showmode D_showmode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_D_showmode;
  /*!
   * WindowEの表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: windowE_showmode E_showmode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_E_showmode;
  /*!
   * WindowFの表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: windowF_showmode F_showmode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_F_showmode;
  /*!
   * WindowGの表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: windowG_showmode G_showmode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_G_showmode;
  /*!
   * Window1の表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * 0:Sサイズ
   * 1:Mサイズ
   * 2:Lサイズ
   * 3:XLサイズ
   * - Name: windowA_showsize A_showsize
   * - DefaultValue: 0
   * - Unit: -
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_A_showsize;
  /*!
   * WindowBの表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * 0:Sサイズ
   * 1:Mサイズ
   * 2:Lサイズ
   * 3:XLサイズ
   * - Name: windowB_showsize B_showsize
   * - DefaultValue: 0
   * - Unit: -
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_B_showsize;
  /*!
   * WindowCの表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * 0:Sサイズ
   * 1:Mサイズ
   * 2:Lサイズ
   * 3:XLサイズ
   * - Name: windowC_showsize C_showsize
   * - DefaultValue: 0
   * - Unit: -
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_C_showsize;
  /*!
   * WindowDの表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * 0:Sサイズ
   * 1:Mサイズ
   * 2:Lサイズ
   * 3:XLサイズ
   * - Name: windowD_showsize D_showsize
   * - DefaultValue: 0
   * - Unit: -
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_D_showsize;
  /*!
   * WindowEの表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * 0:Sサイズ
   * 1:Mサイズ
   * 2:Lサイズ
   * 3:XLサイズ
   * - Name: windowE_showsize E_showsize
   * - DefaultValue: 0
   * - Unit: -
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_E_showsize;
  /*!
   * WindowFの表示サイズの切り替えを指定するパラメータ。
   * 小/中/大の３段階から切り替えを行う。
   * 0:小サイズ
   * 1:中サイズ
   * 2:大サイズ
   * - Name: windowF_showsize F_showsize
   * - DefaultValue: 0
   * - Unit: -
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_F_showsize;
  /*!
   * WindowGの表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * 0:Sサイズ
   * 1:Mサイズ
   * 2:Lサイズ
   * 3:XLサイズ
   * - Name: windowG_showsize G_showsize
   * - DefaultValue: 0
   * - Unit: -
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_G_showsize;
  /*!
   * WindowHの表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * 0:Sサイズ
   * 1:Mサイズ
   * 2:Lサイズ
   * 3:XLサイズ
   * - Name: windowH_showsize H_showsize
   * - DefaultValue: 0
   * - Unit: -
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_H_showsize;
  /*!
   * WindowIの表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * 0:Sサイズ
   * 1:Mサイズ
   * 2:Lサイズ
   * 3:XLサイズ
   * - Name: windowI_showsize I_showsize
   * - DefaultValue: 2
   * - Unit: -
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_I_showsize;
  /*!
   * WindowHの表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: windowH_showmode H_showmode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_H_showmode;
  /*!
   * WindowIの表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: windowI_showmode I_showmode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_I_showmode;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::CameraImage m_inCameraImage;
  /*!
   * CameraImage型のデータを取得する。
   * - Type: RTC::CameraImage
   * - Number: 7（tm/width/height/bpp/format/fDiv/pixels)
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのCamer
   *              aImage型を使用する。
   * - Unit: -
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::CameraImage> m_inCameraImageIn;
  RTC::MultiCameraImages m_inMultiCameraImage;
  /*!
   * MultiCameraImages型のデータを取得する。
   * - Type: RTC::MultiCameraImages
   * - Number: 8 (Camera Id0-7)×
   *           7（tm/width/height/bpp/format/fDiv/pixels)
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのMulti
   *              CameraImages型を使用する。
   * - Unit: -
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::MultiCameraImages> m_inMultiCameraImageIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>
	 static const uWord_Type WINDOW_SIZE_S_W = 160;    // default width(S-size) of application window to create 
	 static const uWord_Type WINDOW_SIZE_S_H = 120;    // default height(S-size) of application window to create 

	 static const uWord_Type WINDOW_SIZE_M_W = 180;    // default width(M-size) of application window to create 
	 static const uWord_Type WINDOW_SIZE_M_H = 135;    // default height(M-size) of application window to create 

	 static const uWord_Type WINDOW_SIZE_L_W = 240;    // default width(L-size) of application window to create 
	 static const uWord_Type WINDOW_SIZE_L_H = 180;    // default height(L-size) of application window to create 

	 static const uWord_Type WINDOW_SIZE_XL_W = 320;   // default width(XL-size) of application window to create 
	 static const uWord_Type WINDOW_SIZE_XL_H = 240;   // default height(XL-size) of application window to create 

	 static const uWord_Type WINDOW_MOVE_X_OFFSET = 260;    // offset of application window 
	 static const uWord_Type WINDOW_MOVE_Y_OFFSET = 200;    // offset of application window 

	 static  const uWord_Type WINDOW_MOVE_X_OFFSET2 = 220;    // offset of application window 
	 static const uWord_Type WINDOW_MOVE_Y_OFFSET2 = 170;    // offset of application window 

	 static const uWord_Type WINDOW_MOVE_X1 = 400;    // default x1 position of application window to create 
	 static const uWord_Type WINDOW_MOVE_X2 = (WINDOW_MOVE_X1 + WINDOW_MOVE_X_OFFSET);    // default x2 position of application window to create 
	 static const uWord_Type WINDOW_MOVE_X3 = (WINDOW_MOVE_X2 + WINDOW_MOVE_X_OFFSET);    // default x3 position of application window to create 
	 static const uWord_Type WINDOW_MOVE_X4 = (WINDOW_MOVE_X1 + WINDOW_MOVE_X_OFFSET2);    // default x4 position of application window to create 

	 static const uWord_Type WINDOW_MOVE_Y1 = 50;    // default y1 position of application window to create 
	 static const uWord_Type WINDOW_MOVE_Y2 = (WINDOW_MOVE_Y1 + WINDOW_MOVE_Y_OFFSET);    // default y2 position of application window to create 
	 static const uWord_Type WINDOW_MOVE_Y3 = (WINDOW_MOVE_Y2 + WINDOW_MOVE_Y_OFFSET);    // default y3 position of application window to create 

	 static const uWord_Type WINDOW_MOVE_Y4 = (WINDOW_MOVE_Y1 + WINDOW_MOVE_Y_OFFSET2);    // default y4 position of application window to create 

	 static const uWord_Type IMAGE_SIZE_W_MAX = 640;    // input cameraimage data maximum width 
	 static const uWord_Type IMAGE_SIZE_H_MAX = 640;    // input cameraimage data maximum height 

	 static const uWord_Type CONF_WINDOW_SHOW_ACT = 1;    // configuration parameter "activate window" 

	 static const uWord_Type CONF_WINDOW_SIZE_M = 1;    // configuration parameter of size "window M-size" 

	 static const uWord_Type ACTIVE_WINDOW_MAX = 9;    // maximum number of window to create 
	 static const uWord_Type CONF_WINDOW_SIZE_MAX = 4;    // configuration parameter maximum value 
	 //static const uWord_Type MULTI_CAMERA_DEVICE_MAX = 8;    // maximum number of Multi Camera device  
	 const uWord_Type DISP_RECOMM_RES_RIGHT = 1366;
	 const uWord_Type DISP_RECOMM_RES_BOTTOM = 768;

	 uWord_Type windowA_showmode;
	 uWord_Type windowB_showmode;
	 uWord_Type windowC_showmode;
	 uWord_Type windowD_showmode;
	 uWord_Type windowE_showmode;
	 uWord_Type windowF_showmode;
	 uWord_Type windowG_showmode;
	 uWord_Type windowH_showmode;
	 uWord_Type windowI_showmode;

	 enum CameraDeviceID{
		 CAMERA_DEVICE_0 = 0,
		 CAMERA_DEVICE_1 = 1,
		 CAMERA_DEVICE_2 = 2,
		 CAMERA_DEVICE_3 = 3,
		 CAMERA_DEVICE_4 = 4,
		 CAMERA_DEVICE_5 = 5,
		 CAMERA_DEVICE_6 = 6,
		 CAMERA_DEVICE_7 = 7,
		 CAMERA_DEVICE_MAX = 8
	 };

	 struct winSize_st
	 {
		 uWord_Type uW_SizeW;    // width of window 
		 uWord_Type uW_SizeH;    // height of window 
	 };

	 struct appWindow_st
	 {
		 std::string winName;    // name of window to create
		 winSize_st *winSizes;   // size of window to create
		 uWord_Type uW_MoveX;    // x position of window to create 
		 uWord_Type uW_MoveY;    // y position of window to create
		 uWord_Type uW_ConfShow;    // configuration to create window 
		 uWord_Type uW_ConfSize;    // configuration to set size of window 
	 };
	 std::string winname_st[CAMERA_DEVICE_MAX];

	 const std::string strWinNameA;    //application window1 name
	 const std::string strWinNameB;    // application window2 name 
	 const std::string strWinNameC;    // application window3 name 
	 const std::string strWinNameD;    // application window4 name 
	 const std::string strWinNameE;    // application window5 name 
	 const std::string strWinNameF;    // application window6 name 
	 const std::string strWinNameG;    // application window7 name 
	 const std::string strWinNameH;    // application window8 name 
	 const std::string strWinNameI;    // application windowZ name 

	 void readCameraImage();    //reading cameraimage data. to view the Image
	 void readMultiCameraImage();    //reading cameraimage data. to view the Image
	 void createAppWindow();    //to create apllication window
	 void showCameraImage(RTC::CameraImage, std::string);    //to view the Image
	 void showMultiCameraImage(RTC::MultiCameraImages);    //to view the Image
	 void closeAppWindow();    //to close apllication window
	 uWord_Type checkImagesize(RTC::CameraImage, std::string);
	 uWord_Type checkMultiImagesize(uWord_Type, uWord_Type, std::string);
	 uWord_Type checkDisplay_resolution();
};


extern "C"
{
  DLL_EXPORT void BirdseyeImageViewerInit(RTC::Manager* manager);
};

#endif // BIRDSEYEIMAGEVIEWER_H
