﻿// -*- C++ -*-
/*!
 * @file  BirdseyeImageViewer.cpp
 * @brief Birdseye Image Viewer component
 * @date $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * $Id$
 */
#include "BirdseyeImageViewer.h"
static int count = 0;
// Module specification
// <rtc-template block="module_spec">
static const char* birdseyeimageviewer_spec[] =
  {
    "implementation_id", "BirdseyeImageViewer",
    "type_name",         "BirdseyeImageViewer",
    "description",       "Birdseye Image Viewer component",
    "version",           "1.0.0",
    "vendor",            "EAC",
    "category",          "Viewer",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.A_showmode", "1",
    "conf.default.B_showmode", "1",
    "conf.default.C_showmode", "1",
    "conf.default.D_showmode", "1",
    "conf.default.E_showmode", "1",
    "conf.default.F_showmode", "1",
    "conf.default.G_showmode", "1",
    "conf.default.A_showsize", "0",
    "conf.default.B_showsize", "0",
    "conf.default.C_showsize", "0",
    "conf.default.D_showsize", "0",
    "conf.default.E_showsize", "0",
    "conf.default.F_showsize", "0",
    "conf.default.G_showsize", "0",
    "conf.default.H_showsize", "0",
    "conf.default.I_showsize", "2",
    "conf.default.H_showmode", "1",
    "conf.default.I_showmode", "1",
    // Widget
    "conf.__widget__.A_showmode", "radio",
    "conf.__widget__.B_showmode", "radio",
    "conf.__widget__.C_showmode", "radio",
    "conf.__widget__.D_showmode", "radio",
    "conf.__widget__.E_showmode", "radio",
    "conf.__widget__.F_showmode", "radio",
    "conf.__widget__.G_showmode", "radio",
    "conf.__widget__.A_showsize", "radio",
    "conf.__widget__.B_showsize", "radio",
    "conf.__widget__.C_showsize", "radio",
    "conf.__widget__.D_showsize", "radio",
    "conf.__widget__.E_showsize", "radio",
    "conf.__widget__.F_showsize", "radio",
    "conf.__widget__.G_showsize", "radio",
    "conf.__widget__.H_showsize", "radio",
    "conf.__widget__.I_showsize", "radio",
    "conf.__widget__.H_showmode", "radio",
    "conf.__widget__.I_showmode", "radio",
    // Constraints
    "conf.__constraints__.A_showmode", "(0,1)",
    "conf.__constraints__.B_showmode", "(0,1)",
    "conf.__constraints__.C_showmode", "(0,1)",
    "conf.__constraints__.D_showmode", "(0,1)",
    "conf.__constraints__.E_showmode", "(0,1)",
    "conf.__constraints__.F_showmode", "(0,1)",
    "conf.__constraints__.G_showmode", "(0,1)",
    "conf.__constraints__.A_showsize", "(0,1,2,3)",
    "conf.__constraints__.B_showsize", "(0,1,2,3)",
    "conf.__constraints__.C_showsize", "(0,1,2,3)",
    "conf.__constraints__.D_showsize", "(0,1,2,3)",
    "conf.__constraints__.E_showsize", "(0,1,2,3)",
    "conf.__constraints__.F_showsize", "(0,1,2,3)",
    "conf.__constraints__.G_showsize", "(0,1,2,3)",
    "conf.__constraints__.H_showsize", "(0,1,2,3)",
    "conf.__constraints__.I_showsize", "(0,1,2,3)",
    "conf.__constraints__.H_showmode", "(0,1)",
    "conf.__constraints__.I_showmode", "(0,1)",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
BirdseyeImageViewer::BirdseyeImageViewer(RTC::Manager* manager)
// <rtc-template block="initializer">
: RTC::DataFlowComponentBase(manager),
	m_inCameraImageIn("inCameraImage", m_inCameraImage),
	m_inMultiCameraImageIn("inMultiCameraImage", m_inMultiCameraImage),
	strWinNameA("画像A"),
	strWinNameB("画像B"),
	strWinNameC("画像C"),
	strWinNameD("画像D"),
	strWinNameE("画像E"),
	strWinNameF("画像F"),
	strWinNameG("画像G"),
	strWinNameH("画像H"),
	strWinNameI("俯瞰合成画像")
	//m_stMultiCapture_p(NULL)
	// </rtc-template>
{
}

/*!
 * @brief destructor
 */
BirdseyeImageViewer::~BirdseyeImageViewer()
{
}


/*!
 * Initialization of Viewer RTC
 */
RTC::ReturnCode_t BirdseyeImageViewer::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inCameraImage", m_inCameraImageIn);
  addInPort("inMultiCameraImage", m_inMultiCameraImageIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("A_showmode", m_A_showmode, "1");
  bindParameter("B_showmode", m_B_showmode, "1");
  bindParameter("C_showmode", m_C_showmode, "1");
  bindParameter("D_showmode", m_D_showmode, "1");
  bindParameter("E_showmode", m_E_showmode, "1");
  bindParameter("F_showmode", m_F_showmode, "1");
  bindParameter("G_showmode", m_G_showmode, "1");
  bindParameter("A_showsize", m_A_showsize, "0");
  bindParameter("B_showsize", m_B_showsize, "0");
  bindParameter("C_showsize", m_C_showsize, "0");
  bindParameter("D_showsize", m_D_showsize, "0");
  bindParameter("E_showsize", m_E_showsize, "0");
  bindParameter("F_showsize", m_F_showsize, "0");
  bindParameter("G_showsize", m_G_showsize, "0");
  bindParameter("H_showsize", m_H_showsize, "0");
  bindParameter("I_showsize", m_I_showsize, "2");
  bindParameter("H_showmode", m_H_showmode, "1");
  bindParameter("I_showmode", m_I_showmode, "1");
  // </rtc-template>
  winname_st[CAMERA_DEVICE_0] = strWinNameA;
  winname_st[CAMERA_DEVICE_1] = strWinNameB;
  winname_st[CAMERA_DEVICE_2] = strWinNameC;
  winname_st[CAMERA_DEVICE_3] = strWinNameD;
  winname_st[CAMERA_DEVICE_4] = strWinNameE;
  winname_st[CAMERA_DEVICE_5] = strWinNameF;
  winname_st[CAMERA_DEVICE_6] = strWinNameG;
  winname_st[CAMERA_DEVICE_7] = strWinNameH;
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t BirdseyeImageViewer::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BirdseyeImageViewer::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BirdseyeImageViewer::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * Activate the Viewer RTC
 */

RTC::ReturnCode_t BirdseyeImageViewer::onActivated(RTC::UniqueId ec_id)
{
  createAppWindow();
  
  checkDisplay_resolution();
  
  m_inMultiCameraImage.images.length(CAMERA_DEVICE_MAX);

  return RTC::RTC_OK;
}

/*!
 * Deactivate the Viewer RTC
 */

RTC::ReturnCode_t BirdseyeImageViewer::onDeactivated(RTC::UniqueId ec_id)
{
  closeAppWindow();
// if (m_stMultiCapture_p != NULL)
// {
    m_inMultiCameraImage.images.release();
// }

  return RTC::RTC_OK;
}

/*!
 * Periodic task of Veiwer RTC. This task read the image file.
 */

RTC::ReturnCode_t BirdseyeImageViewer::onExecute(RTC::UniqueId ec_id)
{
  readCameraImage();
  readMultiCameraImage();
  cv::waitKey(10);
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t BirdseyeImageViewer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BirdseyeImageViewer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BirdseyeImageViewer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BirdseyeImageViewer::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BirdseyeImageViewer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*!
* reading cameraimage data. to view the Image.
*/
void BirdseyeImageViewer::readCameraImage()
{
  if (windowI_showmode == CONF_WINDOW_SHOW_ACT)
  {
    if (m_inCameraImageIn.isNew())
    {
      m_inCameraImageIn.read();
	  showCameraImage(m_inCameraImage, strWinNameI);
	}
  }
}
void BirdseyeImageViewer::readMultiCameraImage()
{
  if (m_inMultiCameraImageIn.isNew())
  {
     m_inMultiCameraImageIn.read();
	 showMultiCameraImage(m_inMultiCameraImage);
  }
}

/*!
* to create apllication window.
*/
void BirdseyeImageViewer::createAppWindow()
{
	winSize_st astWinSizes[CONF_WINDOW_SIZE_MAX] =
	{
		{ WINDOW_SIZE_S_W, WINDOW_SIZE_S_H },
		{ WINDOW_SIZE_M_W, WINDOW_SIZE_M_H },
		{ WINDOW_SIZE_L_W, WINDOW_SIZE_L_H },
		{ WINDOW_SIZE_XL_W, WINDOW_SIZE_XL_H }
	};
	appWindow_st astAppWindows[ACTIVE_WINDOW_MAX] =
	{
		{ strWinNameA, astWinSizes, WINDOW_MOVE_X2, WINDOW_MOVE_Y1, m_A_showmode, m_A_showsize },
		{ strWinNameB, astWinSizes, WINDOW_MOVE_X3, WINDOW_MOVE_Y1, m_B_showmode, m_B_showsize },
		{ strWinNameC, astWinSizes, WINDOW_MOVE_X3, WINDOW_MOVE_Y2, m_C_showmode, m_C_showsize },
		{ strWinNameD, astWinSizes, WINDOW_MOVE_X3, WINDOW_MOVE_Y3, m_D_showmode, m_D_showsize },
		{ strWinNameE, astWinSizes, WINDOW_MOVE_X2, WINDOW_MOVE_Y3, m_E_showmode, m_E_showsize },
		{ strWinNameF, astWinSizes, WINDOW_MOVE_X1, WINDOW_MOVE_Y3, m_F_showmode, m_F_showsize },
		{ strWinNameG, astWinSizes, WINDOW_MOVE_X1, WINDOW_MOVE_Y2, m_G_showmode, m_G_showsize },
		{ strWinNameH, astWinSizes, WINDOW_MOVE_X1, WINDOW_MOVE_Y1, m_H_showmode, m_H_showsize },
		{ strWinNameI, astWinSizes, WINDOW_MOVE_X4, WINDOW_MOVE_Y4, m_I_showmode, m_I_showsize }
	};
	std::cout << "create window sequence bigin" << std::endl;

	windowA_showmode = m_A_showmode;
	windowB_showmode = m_B_showmode;
	windowC_showmode = m_C_showmode;
	windowD_showmode = m_D_showmode;
	windowE_showmode = m_E_showmode;
	windowF_showmode = m_F_showmode;
	windowG_showmode = m_G_showmode;
	windowH_showmode = m_H_showmode;
	windowI_showmode = m_I_showmode;

	uWord_Type auW_Cnt;
	uWord_Type auW_Size = CONF_WINDOW_SIZE_M;

	for (auW_Cnt = 0; auW_Cnt < ACTIVE_WINDOW_MAX; auW_Cnt++)
	{
		if (astAppWindows[auW_Cnt].uW_ConfShow == CONF_WINDOW_SHOW_ACT)
		{
			auW_Size = astAppWindows[auW_Cnt].uW_ConfSize;
			if (auW_Size < CONF_WINDOW_SIZE_MAX)
			{
				std::cout << "create window " << "[" << astAppWindows[auW_Cnt].winName << "]" << "/size:" << auW_Size << std::endl;
				cv::namedWindow(astAppWindows[auW_Cnt].winName, cv::WINDOW_NORMAL);
				cv::resizeWindow(astAppWindows[auW_Cnt].winName, astAppWindows[auW_Cnt].winSizes[auW_Size].uW_SizeW, astAppWindows[auW_Cnt].winSizes[auW_Size].uW_SizeH);
				cv::moveWindow(astAppWindows[auW_Cnt].winName, astAppWindows[auW_Cnt].uW_MoveX, astAppWindows[auW_Cnt].uW_MoveY);
			}
		}
		else
		{
			std::cout << "not create window " << "[" << astAppWindows[auW_Cnt].winName << "]" << std::endl;
		}
	}
	std::cout << "create window sequence end" << std::endl;
}
/*!
* to view the Image.
*/
void BirdseyeImageViewer::showCameraImage(RTC::CameraImage image, std::string cam_image)
{
  static coil::TimeValue tm_pre;
  //static int count = 0;

  if (checkImagesize(image, cam_image))
  {
    cv::Mat m_inimagebuf = cv::Mat(cv::Size(image.width, image.height), CV_8UC3);
    memcpy(m_inimagebuf.data, (void *)&(image.pixels[0]), image.pixels.length());
	cv::namedWindow(cam_image, cv::WINDOW_NORMAL);
	cv::imshow(cam_image, m_inimagebuf);
    //cv::waitKey(1);

#ifdef _DEBUG
//		std::cout << "[CameraImg]" << std::endl;
//		std::cout << "width:" << image.width << std::endl;
//		std::cout << "height:" << image.height << std::endl;
//		std::cout << cam_image << ":rows:" << m_inimagebuf.rows << std::endl;
//		std::cout << cam_image << ":cols:" << m_inimagebuf.cols << std::endl;
//		std::cout << cam_image << ":channels:" << m_inimagebuf.channels() << std::endl;
//		std::cout << cam_image << ":depth:" << m_inimagebuf.rows << std::endl;
//		std::cout << cam_image << ":elememSize1:" << m_inimagebuf.elemSize1() << std::endl;

		if (count > 100)
		{
			count = 0;
			coil::TimeValue tm;
			tm = coil::gettimeofday();
			double sec(tm - tm_pre);

			if (sec > 1.0 && sec < 1000.0)
			{
				std::cout << 100.0 / sec << " [FPS]" << std::endl;
			}
			tm_pre = tm;
		}

		++count;

#endif
  }
}

/*!
* to view the MultiCameraImage.
*/
void BirdseyeImageViewer::showMultiCameraImage(RTC::MultiCameraImages Multiimage)
{
	uWord_Type imgwidth;
	uWord_Type imgheight;
	uWord_Type imglen;

	uWord_Type stConfCheck[CAMERA_DEVICE_MAX] =
	{
		windowA_showmode,
		windowB_showmode,
		windowC_showmode,
		windowD_showmode,
		windowE_showmode,
		windowF_showmode,
		windowG_showmode,
		windowH_showmode
	};

	uWord_Type i = 0;

	//std::string cam_image;

	for (i = 0; i < CAMERA_DEVICE_MAX; i++)
	{
		if (stConfCheck[i] == CONF_WINDOW_SHOW_ACT)
		{
			imgwidth = Multiimage.images[i].width;
			imgheight = Multiimage.images[i].height;
			imglen = Multiimage.images[i].pixels.length();

			if (checkMultiImagesize(imgwidth, imgheight, winname_st[i]))
			{
				cv::Mat m_inimagebuf = cv::Mat(cv::Size(imgwidth, imgheight), CV_8UC3);
				memcpy(m_inimagebuf.data, (void *)&(Multiimage.images[i].pixels[0]), imglen);
				cv::namedWindow(winname_st[i], cv::WINDOW_NORMAL);
				cv::imshow(winname_st[i], m_inimagebuf);
				//cv::waitKey(1);
			}
		}
	}
}

/*!
* to close apllication window
*/
void BirdseyeImageViewer::closeAppWindow()
{
	std::cout << "destroy all windows" << std::endl;
	cv::destroyAllWindows();
}

/*!
* Camera Image data size check
*/
uWord_Type BirdseyeImageViewer::checkImagesize(RTC::CameraImage image, std::string cam_image)
{
	uWord_Type retVal;
	retVal = false;

	if ((image.width == 0) || (image.height == 0))
	{
		std::cerr << "[" << cam_image << "]" << "input CameraImage data is empty!" << std::endl;
	}
	else if ((image.width <= IMAGE_SIZE_W_MAX) && (image.height <= IMAGE_SIZE_H_MAX))
	{
		retVal = true;
	}
	else
	{
		std::cerr << "[" << cam_image << "]" << "input CameraImage data is oversized!" << std::endl;
	}
	return retVal;
}

/*!
* Multi Camera Image data size check
*/
uWord_Type BirdseyeImageViewer::checkMultiImagesize(uWord_Type imgwidth, uWord_Type imgheight, std::string cam_image)
{
	uWord_Type retVal;
	retVal = false;

	if ((imgwidth == 0) || (imgheight == 0))
	{
		std::cerr << "[" << cam_image << "]" << "input CameraImage data is empty!" << std::endl;
	}
	else if ((imgwidth <= IMAGE_SIZE_W_MAX) && (imgheight <= IMAGE_SIZE_H_MAX))
	{
		retVal = true;
	}
	else
	{
		std::cerr << "[" << cam_image << "]" << "input CameraImage data is oversized!" << std::endl;
	}
	return retVal;
}

/*!
* Dislpay resolution check
*/
uWord_Type BirdseyeImageViewer::checkDisplay_resolution()
{
	uWord_Type retVal;
	RECT rc;
	retVal = true;

	GetWindowRect(GetDesktopWindow(), &rc);
#ifdef _DEBUG
	std::cout << "GetWindowRect:" << rc.right << "/" << rc.bottom << std::endl;
#endif
	if ((rc.right < DISP_RECOMM_RES_RIGHT) || (rc.bottom < DISP_RECOMM_RES_BOTTOM))
	{
		retVal = false;
		std::cout << "Display resolution is not enough." << std::endl;
		std::cout << "some Images may not display." << std::endl;
	}
	return retVal;
}

extern "C"
{
 
  void BirdseyeImageViewerInit(RTC::Manager* manager)
  {
    coil::Properties profile(birdseyeimageviewer_spec);
    manager->registerFactory(profile,
                             RTC::Create<BirdseyeImageViewer>,
                             RTC::Delete<BirdseyeImageViewer>);
  }
  
};
